package com.mall.gateway.filters;
import com.mall.common.constant.TokenConstants;
import com.mall.common.redis.RedisCache;
import com.mall.common.utils.JwtUtils;
import com.mall.common.utils.StringUtils;
import com.mall.gateway.config.IgnoreWhiteConfig;
import com.mall.gateway.utils.GatewayUtils;
import io.jsonwebtoken.Claims;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

/**
 * @description: 鉴权过滤器
 * @author DongZl
 */
@Component
@Log4j2
public class AuthFilter implements GlobalFilter, Ordered {
    /**
     * redis操作
     */
    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 请求作用域
        ServerHttpRequest request = exchange.getRequest();
        // 请求头
        HttpHeaders headers = request.getHeaders();
        // 请求方式
        HttpMethod method = request.getMethod();

        String uri = request.getURI().getPath();

        log.info("请求日志：uri:[{}] , 请求方式:[{}]", uri, method);
        // 跳过不需要验证的路径
        if (StringUtils.matches(uri, ignoreWhiteConfig.getWhites())) {
            return chain.filter(exchange);
        }

        String token = headers.getFirst(TokenConstants.TOKEN);
        if (StringUtils.isEmpty(token)) {
            return GatewayUtils.errorResponse(exchange, "令牌不能为空");
        }

        Claims claims = JwtUtils.parseToken(token);
        if (claims == null) {
            return GatewayUtils.errorResponse(exchange, "令牌已过期或验证不正确！");
        }

        boolean login = redisCache.hasKey(TokenConstants.TOKEN + token);
        if (!login) {
            return GatewayUtils.errorResponse(exchange, "登录状态已过期");
        }
        return chain.filter(exchange);
    }
    @Override
    public int getOrder() {
        return 0;
    }
}
